/*

    this source file is a part of 
    lp_parse - a preprocessor for MILP models
    Copyright (C) 1996  Ari Jolma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    for more information please contact 
    Ari Jolma
    Helsinki University of Technology
    Laboratory of Water Resources Engineering
    Building W
    02150 Espoo, Finland
    E-mail: Ari.Jolma@hut.fi

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "util.h"
#include "lp_out.h"

XFILE *lps_infile = NULL;
XFILE *lp_outfile = NULL;

char OperatorBuffer = '\0';
double NumberBuffer;
int NumberBufferUsed = FALSE;
int HoldingNumber = FALSE;
char CalculatorBuffer;

void OperatorOut(void) {
  if (OperatorBuffer) {
    switch (OperatorBuffer) {
    case ';':;
    case '+': fprintf(lp_outfile->f, "%c\n", OperatorBuffer); break;
    default: fprintf(lp_outfile->f, "%c", OperatorBuffer);
    }
    OperatorBuffer = '\0';
  }
}

void NumberOut(void) {
  if (NumberBufferUsed) {
    if ((NumberBuffer < 0) && 
	((OperatorBuffer == '+') || 
	 (OperatorBuffer == '-')))
      OperatorBuffer = '\n';
    OperatorOut();
    fprintf(lp_outfile->f, "%f", NumberBuffer);
    NumberBufferUsed = FALSE;
  }
}

void streamer_flush(void) {
  if (HoldingNumber) {
    NumberOut();
    OperatorBuffer = CalculatorBuffer;
    HoldingNumber = FALSE;
  }
  OperatorOut();
}

void stream_out_token(char *token) {
  streamer_flush();
  fprintf(lp_outfile->f, "%s", token);
}

void stream_out_char(char c) {
  if (((c == '*') || (c == '/')) && NumberBufferUsed) {
    HoldingNumber = TRUE;
    CalculatorBuffer = c;
  } else {
    NumberOut();
    OperatorBuffer = c;
  }
}

void stream_out_number(double x) {
  if (HoldingNumber) {
    switch (CalculatorBuffer) {
    case '*': NumberBuffer *= x; break;
    case '/': NumberBuffer /= x; break;
    }
    HoldingNumber = FALSE;
  } else {
    NumberBuffer = x;
    NumberBufferUsed = TRUE;
  }
}

