/*

    this source file is a part of 
    lp_parse - a preprocessor for MILP models
    Copyright (C) 1996  Ari Jolma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    for more information please contact 
    Ari Jolma
    Helsinki University of Technology
    Laboratory of Water Resources Engineering
    Building W
    02150 Espoo, Finland
    E-mail: Ari.Jolma@hut.fi

 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "patchlev.h"
#include "util.h"
#include "lp_out.h"
#include "lp_index.h"
#include "lp_parse.h"

void print_help(void) {
  printf("lp_parse version %s (c) 1996 Ari Jolma (ajolma@ahti.hut.fi)\n", PATCHLEVEL);
  printf("Usage:\n");
  printf("lp_parse [flags] [f1] [f2]\n");
  printf("flags (do not mix):\n");
  printf("  -?     print this help\n");
  printf("  --help print this help\n");
  printf("  -v  print version number\n");
  printf("  -q  quiet (no warnings of undeclared variables)\n");
  printf("f1 = lps model\n");
  printf("f2 = lp model\n");
  printf("if f1 is not given, the lps model is read from stdin\n");
  printf("if f2 is not given, the lp model is output to stdout\n");  
  printf("data for the model is read by default from stdin\n");
  exit(0);
}

int main(int argc, char *argv[]) {
  int i;
  pn = filename(argv[0]);
  for(i = 1; i < argc; i++) {
    if (strcmp(argv[i], "-?") == 0) print_help();
    else if (strcmp(argv[i], "--help") == 0) print_help();
    else if (strcmp(argv[i], "-v") == 0) {
      printf("Version %s\n",PATCHLEVEL);
      exit(0);
    }
    else if (strcmp(argv[i], "-q") == 0) lps_quiet = TRUE;
    else if (!lps_infile) {
      lps_infile = open_file(argv[i], "r");
      if (!lps_infile) exit(1);
    }
    else if (!lp_outfile) {
      lp_outfile = open_file(argv[i], "w");
      if (!lp_outfile) exit(1);
    }
  }
  if (!lps_infile) lps_infile = xstdin();
  if (!lps_infile) exit(1);
  if (!lp_outfile) lp_outfile = xstdout();
  if (!parse_lps()) exit(1);
  return 0;
}
