/*

    this source file is a part of 
    lp_parse - a preprocessor for MILP models
    Copyright (C) 1996  Ari Jolma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    for more information please contact 
    Ari Jolma
    Helsinki University of Technology
    Laboratory of Water Resources Engineering
    Building W
    02150 Espoo, Finland
    E-mail: Ari.Jolma@hut.fi

 */

#define TRUE 1
#define FALSE 0

extern FILE *mystderr;
extern char *pn;

/* --------------------------------------------------------------------------*/

void *xmalloc(size_t size);
void *xrealloc(void *ptr, size_t size);
void *xcalloc(size_t nmemb, size_t size);

#define XMS(S) (S *)xmalloc(sizeof(S))

/* --------------------------------------------------------------------------*/

#define SSCANF_INT(s, i, r, xf) \
  { if (s) \
    { if (sscanf(s, "%i", &(i)) < 1) \
      { fprintf(mystderr, "%s:%s:%i: \"%s\" is not an integer\n", \
                pn, filename(xf->name), xf->cln, (s)); return r; } \
    } else (i) = 0; }

#define SSCANF_DOUBLE(s, f, r, xf) \
  { if (s) { \
    if (sscanf(s, "%lf", &(f)) < 1) { \
    fprintf(mystderr, "%s:%s:%i: \"%s\" is not a number\n", \
            pn, filename(xf->name), xf->cln, (s)); \
    return r; \
  } \
} else (f) = 0; }

/* --------------------------------------------------------------------------*/

/*
#define WHITESPACE(s) (((s) == ' ') || ((s) == '\t') || ((s) == '\r') || ((s) == '\n'))
#define SKIP_WHITESPACE(s) { while (WHITESPACE(*s) && (*(s) != '\0')) (s)++; }
*/

/* change s1, s2 has test-string */
#define SKIP_UNTIL(s1,s2) { while ((*(s1) != '\0') && (strchr((s2),(*(s1))) == NULL)) (s1)++; }
#define SKIP_UNTIL_NOT(s1,s2) { while ((*(s1) != '\0') && (strchr((s2),(*(s1))) != NULL)) (s1)++; }

/* --------------------------------------------------------------------------*/

/* class_number is a string of numbers, the rightmost is the most significant
		 this function tests if Sub is a descendant of Super */

#define CLASS_NUMBER_BASE 0x0010
#define CNR unsigned long

int IsSubClass(CNR Super, CNR Sub);

/* n should be the size of cns and cnrs 
   when returning, n has an index to cns/cnrs  */

CNR getcnr(char *cn, char **cns, CNR *cnrs, int *n);

char *getcn(CNR cnr, char **cns, CNR *cnrs, int *n);

/* --------------------------------------------------------------------------*/

char *xstrdup(const char *c);

/* --------------------------------------------------------------------------*/

struct xfile {
  FILE *f;
  char *name;
  int cln;
  int maxlines;
  int lines;
  char **data;
};

#define XFILE struct xfile

XFILE *new_XFILE(char *name);

/* returns NULL if eof */
char *xgetline(XFILE *f, char *buf, int buf_size);

#define MAX_LINE_LEN 2049
#define LINE_CONTINUATION_CHARS "\\_"

/* returns NULL if eof */
char *getextline(XFILE *f, char *buf, int buf_size);

/* --------------------------------------------------------------------------*/

char *filename(char *path);

XFILE *xstdin();
XFILE *xstdout();

XFILE *open_file(char *name, char *flags);


